# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .kyc_get_params import KYCGetParams as KYCGetParams
from .kyc_get_response import KYCGetResponse as KYCGetResponse
from .kyc_create_params import KYCCreateParams as KYCCreateParams
from .kyc_update_params import KYCUpdateParams as KYCUpdateParams
from .account_get_params import AccountGetParams as AccountGetParams
from .kyc_create_response import KYCCreateResponse as KYCCreateResponse
from .kyc_update_response import KYCUpdateResponse as KYCUpdateResponse
from .account_get_response import AccountGetResponse as AccountGetResponse
from .onramp_create_params import OnrampCreateParams as OnrampCreateParams
from .account_create_params import AccountCreateParams as AccountCreateParams
from .offramp_create_params import OfframpCreateParams as OfframpCreateParams
from .onramp_create_response import OnrampCreateResponse as OnrampCreateResponse
from .account_create_response import AccountCreateResponse as AccountCreateResponse
from .offramp_create_response import OfframpCreateResponse as OfframpCreateResponse



import json
from typing import Any, Dict, Optional, cast
from typing_extensions import override

import httpx

from .authorization_signatures import get_authorization_signature


class PrivyHTTPClient(httpx.Client):
    """A custom HTTP client that adds authorization signatures to requests."""

    _authorization_key: Optional[str]

    def __init__(
        self,
        *,
        app_id: str,
        authorization_key: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        """Initialize the client.

        Args:
            app_id: The Privy app ID
            authorization_key: The authorization private key. If not provided, requests will not be signed.
            **kwargs: Additional arguments to pass to httpx.Client
        """
        super().__init__(**kwargs)
        self.app_id = app_id
        self._authorization_key = None

        if authorization_key is not None:
            # Remove the 'wallet-auth:' prefix if present
            self._authorization_key = authorization_key.replace("wallet-auth:", "")

    def _prepare_request(self, request: httpx.Request) -> None:
        """Add authorization signature to the request if authorization_key is set.

        Args:
            request: The request to prepare
        """
        # Skip if no authorization key or not a POST request
        if self._authorization_key is None or request.method != "POST":
            return

        # Get the request body
        try:
            body_str = request.read().decode("utf-8")
            if body_str:
                body = json.loads(body_str)
            else:
                body = {}
        except Exception:
            body = {}

        # Generate the signature
        signature = get_authorization_signature(
            url=str(request.url),
            body=cast(Dict[str, Any], body),
            method=request.method,
            app_id=self.app_id,
            private_key=self._authorization_key,
        )

        # Add the signature to the request headers
        request.headers["privy-authorization-signature"] = signature

    @override
    def send(self, request: httpx.Request, **kwargs: Any) -> httpx.Response:
        """Send a request with authorization signature if authorization_key is set.

        Args:
            request: The request to send
            **kwargs: Additional arguments to pass to httpx.Client.send

        Returns:
            The response from the server
        """
        self._prepare_request(request)
        return super().send(request, **kwargs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["KeyQuorumCreateParams"]


class KeyQuorumCreateParams(TypedDict, total=False):
    public_keys: Required[List[str]]

    authorization_threshold: float

    display_name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal

import httpx

from ..types import policy_create_params, policy_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, strip_not_given, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.policy import Policy

__all__ = ["PoliciesResource", "AsyncPoliciesResource"]


class PoliciesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return PoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return PoliciesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        chain_type: Literal["ethereum"],
        name: str,
        rules: Iterable[policy_create_params.Rule],
        version: Literal["1.0"],
        owner: Optional[policy_create_params.Owner] | NotGiven = NOT_GIVEN,
        owner_id: Optional[str] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Policy:
        """
        Create a new policy.

        Args:
          chain_type: Chain type the policy applies to.

          name: Name to assign to policy.

          rules: The rules that apply to each method the policy covers.

          version: Version of the policy. Currently, 1.0 is the only version.

          owner: The pem-formatted, P-256 public key of the owner of the policy. If you provide
              this, do not specify an owner_id as it will be generated automatically.

          owner_id: The key quorum ID to set as the owner of the policy. If you provide this, do not
              specify an owner.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return self._post(
            "/v1/policies",
            body=maybe_transform(
                {
                    "chain_type": chain_type,
                    "name": name,
                    "rules": rules,
                    "version": version,
                    "owner": owner,
                    "owner_id": owner_id,
                },
                policy_create_params.PolicyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Policy,
        )

    def update(
        self,
        policy_id: str,
        *,
        name: str | NotGiven = NOT_GIVEN,
        rules: Iterable[policy_update_params.Rule] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Policy:
        """
        Update a policy by policy ID.

        Args:
          name: Name to assign to policy.

          rules: The rules that apply to each method the policy covers.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return self._patch(
            f"/v1/policies/{policy_id}",
            body=maybe_transform(
                {
                    "name": name,
                    "rules": rules,
                },
                policy_update_params.PolicyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Policy,
        )

    def delete(
        self,
        policy_id: str,
        *,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Policy:
        """
        Delete a policy by policy ID.

        Args:
          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return self._delete(
            f"/v1/policies/{policy_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Policy,
        )

    def get(
        self,
        policy_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Policy:
        """
        Get a policy by policy ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return self._get(
            f"/v1/policies/{policy_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Policy,
        )


class AsyncPoliciesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncPoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return AsyncPoliciesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        chain_type: Literal["ethereum"],
        name: str,
        rules: Iterable[policy_create_params.Rule],
        version: Literal["1.0"],
        owner: Optional[policy_create_params.Owner] | NotGiven = NOT_GIVEN,
        owner_id: Optional[str] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Policy:
        """
        Create a new policy.

        Args:
          chain_type: Chain type the policy applies to.

          name: Name to assign to policy.

          rules: The rules that apply to each method the policy covers.

          version: Version of the policy. Currently, 1.0 is the only version.

          owner: The pem-formatted, P-256 public key of the owner of the policy. If you provide
              this, do not specify an owner_id as it will be generated automatically.

          owner_id: The key quorum ID to set as the owner of the policy. If you provide this, do not
              specify an owner.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return await self._post(
            "/v1/policies",
            body=await async_maybe_transform(
                {
                    "chain_type": chain_type,
                    "name": name,
                    "rules": rules,
                    "version": version,
                    "owner": owner,
                    "owner_id": owner_id,
                },
                policy_create_params.PolicyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Policy,
        )

    async def update(
        self,
        policy_id: str,
        *,
        name: str | NotGiven = NOT_GIVEN,
        rules: Iterable[policy_update_params.Rule] | NotGiven = NOT_GIVEN,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Policy:
        """
        Update a policy by policy ID.

        Args:
          name: Name to assign to policy.

          rules: The rules that apply to each method the policy covers.

          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return await self._patch(
            f"/v1/policies/{policy_id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "rules": rules,
                },
                policy_update_params.PolicyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Policy,
        )

    async def delete(
        self,
        policy_id: str,
        *,
        privy_authorization_signature: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Policy:
        """
        Delete a policy by policy ID.

        Args:
          privy_authorization_signature: Request authorization signature. If multiple signatures are required, they
              should be comma separated.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        extra_headers = {
            **strip_not_given({"privy-authorization-signature": privy_authorization_signature}),
            **(extra_headers or {}),
        }
        return await self._delete(
            f"/v1/policies/{policy_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Policy,
        )

    async def get(
        self,
        policy_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Policy:
        """
        Get a policy by policy ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return await self._get(
            f"/v1/policies/{policy_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Policy,
        )


class PoliciesResourceWithRawResponse:
    def __init__(self, policies: PoliciesResource) -> None:
        self._policies = policies

        self.create = to_raw_response_wrapper(
            policies.create,
        )
        self.update = to_raw_response_wrapper(
            policies.update,
        )
        self.delete = to_raw_response_wrapper(
            policies.delete,
        )
        self.get = to_raw_response_wrapper(
            policies.get,
        )


class AsyncPoliciesResourceWithRawResponse:
    def __init__(self, policies: AsyncPoliciesResource) -> None:
        self._policies = policies

        self.create = async_to_raw_response_wrapper(
            policies.create,
        )
        self.update = async_to_raw_response_wrapper(
            policies.update,
        )
        self.delete = async_to_raw_response_wrapper(
            policies.delete,
        )
        self.get = async_to_raw_response_wrapper(
            policies.get,
        )


class PoliciesResourceWithStreamingResponse:
    def __init__(self, policies: PoliciesResource) -> None:
        self._policies = policies

        self.create = to_streamed_response_wrapper(
            policies.create,
        )
        self.update = to_streamed_response_wrapper(
            policies.update,
        )
        self.delete = to_streamed_response_wrapper(
            policies.delete,
        )
        self.get = to_streamed_response_wrapper(
            policies.get,
        )


class AsyncPoliciesResourceWithStreamingResponse:
    def __init__(self, policies: AsyncPoliciesResource) -> None:
        self._policies = policies

        self.create = async_to_streamed_response_wrapper(
            policies.create,
        )
        self.update = async_to_streamed_response_wrapper(
            policies.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            policies.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            policies.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AccountGetParams"]


class AccountGetParams(TypedDict, total=False):
    provider: Required[Literal["bridge", "bridge-sandbox"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "KYCUpdateParams",
    "Variant0",
    "Variant0Data",
    "Variant0DataIdentifyingInformation",
    "Variant0DataResidentialAddress",
    "Variant0DataDocument",
    "Variant0DataTransliteratedResidentialAddress",
    "Variant1",
    "Variant1Data",
    "Variant1DataIdentifyingInformation",
    "Variant1DataResidentialAddress",
    "Variant1DataDocument",
    "Variant1DataTransliteratedResidentialAddress",
]


class Variant0(TypedDict, total=False):
    data: Required[Variant0Data]

    provider: Required[Literal["bridge"]]


class Variant0DataIdentifyingInformation(TypedDict, total=False):
    issuing_country: Required[str]

    type: Required[str]

    description: str

    expiration: str

    image_back: str

    image_front: str

    number: str


class Variant0DataResidentialAddress(TypedDict, total=False):
    city: Required[str]

    country: Required[str]

    street_line_1: Required[str]

    subdivision: Required[str]

    postal_code: str

    street_line_2: str


class Variant0DataDocument(TypedDict, total=False):
    file: Required[str]

    purposes: Required[List[str]]

    description: str


class Variant0DataTransliteratedResidentialAddress(TypedDict, total=False):
    city: Required[str]

    country: Required[str]

    street_line_1: Required[str]

    subdivision: Required[str]

    postal_code: str

    street_line_2: str


class Variant0Data(TypedDict, total=False):
    birth_date: Required[str]

    email: Required[str]

    first_name: Required[str]

    identifying_information: Required[Iterable[Variant0DataIdentifyingInformation]]

    last_name: Required[str]

    residential_address: Required[Variant0DataResidentialAddress]

    type: Required[Literal["individual"]]

    account_purpose: str

    account_purpose_other: str

    acting_as_intermediary: str

    completed_customer_safety_check_at: str

    documents: Iterable[Variant0DataDocument]

    employment_status: str

    endorsements: List[str]

    expected_monthly_payments_usd: str

    has_signed_terms_of_service: bool

    middle_name: str

    most_recent_occupation: str

    nationality: str

    phone: str

    signed_agreement_id: str

    source_of_funds: str

    transliterated_first_name: str

    transliterated_last_name: str

    transliterated_middle_name: str

    transliterated_residential_address: Variant0DataTransliteratedResidentialAddress

    verified_selfie_at: str


class Variant1(TypedDict, total=False):
    data: Required[Variant1Data]

    provider: Required[Literal["bridge-sandbox"]]


class Variant1DataIdentifyingInformation(TypedDict, total=False):
    issuing_country: Required[str]

    type: Required[str]

    description: str

    expiration: str

    image_back: str

    image_front: str

    number: str


class Variant1DataResidentialAddress(TypedDict, total=False):
    city: Required[str]

    country: Required[str]

    street_line_1: Required[str]

    subdivision: Required[str]

    postal_code: str

    street_line_2: str


class Variant1DataDocument(TypedDict, total=False):
    file: Required[str]

    purposes: Required[List[str]]

    description: str


class Variant1DataTransliteratedResidentialAddress(TypedDict, total=False):
    city: Required[str]

    country: Required[str]

    street_line_1: Required[str]

    subdivision: Required[str]

    postal_code: str

    street_line_2: str


class Variant1Data(TypedDict, total=False):
    birth_date: Required[str]

    email: Required[str]

    first_name: Required[str]

    identifying_information: Required[Iterable[Variant1DataIdentifyingInformation]]

    last_name: Required[str]

    residential_address: Required[Variant1DataResidentialAddress]

    type: Required[Literal["individual"]]

    account_purpose: str

    account_purpose_other: str

    acting_as_intermediary: str

    completed_customer_safety_check_at: str

    documents: Iterable[Variant1DataDocument]

    employment_status: str

    endorsements: List[str]

    expected_monthly_payments_usd: str

    has_signed_terms_of_service: bool

    middle_name: str

    most_recent_occupation: str

    nationality: str

    phone: str

    signed_agreement_id: str

    source_of_funds: str

    transliterated_first_name: str

    transliterated_last_name: str

    transliterated_middle_name: str

    transliterated_residential_address: Variant1DataTransliteratedResidentialAddress

    verified_selfie_at: str


KYCUpdateParams: TypeAlias = Union[Variant0, Variant1]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["KYCUpdateResponse"]


class KYCUpdateResponse(BaseModel):
    status: Literal[
        "not_found",
        "active",
        "awaiting_questionnaire",
        "awaiting_ubo",
        "incomplete",
        "not_started",
        "offboarded",
        "paused",
        "rejected",
        "under_review",
    ]

    user_id: str

    provider_user_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union

from .._models import BaseModel

__all__ = ["UserCreateCustomMetadataResponse"]


class UserCreateCustomMetadataResponse(BaseModel):
    custom_metadata: Dict[str, Union[str, float, bool]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .wallet import Wallet
from .._models import BaseModel

__all__ = ["WalletAuthenticateWithJwtResponse", "EncryptedAuthorizationKey"]


class EncryptedAuthorizationKey(BaseModel):
    ciphertext: str
    """
    The encrypted authorization key corresponding to the user's current
    authentication session.
    """

    encapsulated_key: str
    """Base64-encoded ephemeral public key used in the HPKE encryption process.

    Required for decryption.
    """

    encryption_type: Literal["HPKE"]
    """The encryption type used. Currently only supports HPKE."""


class WalletAuthenticateWithJwtResponse(BaseModel):
    encrypted_authorization_key: EncryptedAuthorizationKey
    """The encrypted authorization key data."""

    expires_at: float
    """The expiration time of the authorization key in seconds since the epoch."""

    wallets: List[Wallet]
    """The wallets that the signer has access to."""



from typing import Union


class PrivyAPIExtension:
    _authorization_key: Union[str, None] = None

    def update_authorization_key(self, authorization_key: str) -> None:
        """Update the authorization key for the PrivyAPI client.

        Args:
          authorization_key: The new authorization key.
        """
        self._authorization_key = authorization_key.replace("wallet-auth:", "")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["AccountCreateParams", "Account", "Address", "Iban", "Swift", "SwiftAccount", "SwiftAddress"]


class AccountCreateParams(TypedDict, total=False):
    account_owner_name: Required[str]

    currency: Required[Literal["usd", "eur"]]

    provider: Required[Literal["bridge", "bridge-sandbox"]]

    account: Account

    address: Address

    bank_name: str

    first_name: str

    iban: Iban

    last_name: str

    swift: Swift


class Account(TypedDict, total=False):
    account_number: Required[str]

    routing_number: Required[str]

    checking_or_savings: Literal["checking", "savings"]


class Address(TypedDict, total=False):
    city: Required[str]

    country: Required[str]

    street_line_1: Required[str]

    postal_code: str

    state: str

    street_line_2: str


class Iban(TypedDict, total=False):
    account_number: Required[str]

    bic: Required[str]

    country: Required[str]


class SwiftAccount(TypedDict, total=False):
    account_number: Required[str]

    bic: Required[str]

    country: Required[str]


class SwiftAddress(TypedDict, total=False):
    city: Required[str]

    country: Required[str]

    street_line_1: Required[str]

    postal_code: str

    state: str

    street_line_2: str


class Swift(TypedDict, total=False):
    account: Required[SwiftAccount]

    address: Required[SwiftAddress]

    category: Required[Literal["client", "parent_company", "subsidiary", "supplier"]]

    purpose_of_funds: Required[List[Literal["intra_group_transfer", "invoice_for_goods_and_services"]]]

    short_business_description: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from privy import PrivyAPI, AsyncPrivyAPI
from tests.utils import assert_matches_type
from privy.types.fiat import AccountGetResponse, AccountCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAccounts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: PrivyAPI) -> None:
        account = client.fiat.accounts.create(
            user_id="user_id",
            account_owner_name="xxx",
            currency="usd",
            provider="bridge",
        )
        assert_matches_type(AccountCreateResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: PrivyAPI) -> None:
        account = client.fiat.accounts.create(
            user_id="user_id",
            account_owner_name="xxx",
            currency="usd",
            provider="bridge",
            account={
                "account_number": "x",
                "routing_number": "xxxxxxxxx",
                "checking_or_savings": "checking",
            },
            address={
                "city": "x",
                "country": "xxx",
                "street_line_1": "x",
                "postal_code": "x",
                "state": "x",
                "street_line_2": "x",
            },
            bank_name="xxx",
            first_name="x",
            iban={
                "account_number": "x",
                "bic": "x",
                "country": "xxx",
            },
            last_name="x",
            swift={
                "account": {
                    "account_number": "x",
                    "bic": "x",
                    "country": "xxx",
                },
                "address": {
                    "city": "x",
                    "country": "xxx",
                    "street_line_1": "x",
                    "postal_code": "x",
                    "state": "x",
                    "street_line_2": "x",
                },
                "category": "client",
                "purpose_of_funds": ["intra_group_transfer"],
                "short_business_description": "x",
            },
        )
        assert_matches_type(AccountCreateResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: PrivyAPI) -> None:
        response = client.fiat.accounts.with_raw_response.create(
            user_id="user_id",
            account_owner_name="xxx",
            currency="usd",
            provider="bridge",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountCreateResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: PrivyAPI) -> None:
        with client.fiat.accounts.with_streaming_response.create(
            user_id="user_id",
            account_owner_name="xxx",
            currency="usd",
            provider="bridge",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountCreateResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_create(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_id` but received ''"):
            client.fiat.accounts.with_raw_response.create(
                user_id="",
                account_owner_name="xxx",
                currency="usd",
                provider="bridge",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_get(self, client: PrivyAPI) -> None:
        account = client.fiat.accounts.get(
            user_id="user_id",
            provider="bridge",
        )
        assert_matches_type(AccountGetResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get(self, client: PrivyAPI) -> None:
        response = client.fiat.accounts.with_raw_response.get(
            user_id="user_id",
            provider="bridge",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountGetResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get(self, client: PrivyAPI) -> None:
        with client.fiat.accounts.with_streaming_response.get(
            user_id="user_id",
            provider="bridge",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountGetResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_id` but received ''"):
            client.fiat.accounts.with_raw_response.get(
                user_id="",
                provider="bridge",
            )


class TestAsyncAccounts:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncPrivyAPI) -> None:
        account = await async_client.fiat.accounts.create(
            user_id="user_id",
            account_owner_name="xxx",
            currency="usd",
            provider="bridge",
        )
        assert_matches_type(AccountCreateResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncPrivyAPI) -> None:
        account = await async_client.fiat.accounts.create(
            user_id="user_id",
            account_owner_name="xxx",
            currency="usd",
            provider="bridge",
            account={
                "account_number": "x",
                "routing_number": "xxxxxxxxx",
                "checking_or_savings": "checking",
            },
            address={
                "city": "x",
                "country": "xxx",
                "street_line_1": "x",
                "postal_code": "x",
                "state": "x",
                "street_line_2": "x",
            },
            bank_name="xxx",
            first_name="x",
            iban={
                "account_number": "x",
                "bic": "x",
                "country": "xxx",
            },
            last_name="x",
            swift={
                "account": {
                    "account_number": "x",
                    "bic": "x",
                    "country": "xxx",
                },
                "address": {
                    "city": "x",
                    "country": "xxx",
                    "street_line_1": "x",
                    "postal_code": "x",
                    "state": "x",
                    "street_line_2": "x",
                },
                "category": "client",
                "purpose_of_funds": ["intra_group_transfer"],
                "short_business_description": "x",
            },
        )
        assert_matches_type(AccountCreateResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.fiat.accounts.with_raw_response.create(
            user_id="user_id",
            account_owner_name="xxx",
            currency="usd",
            provider="bridge",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountCreateResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.fiat.accounts.with_streaming_response.create(
            user_id="user_id",
            account_owner_name="xxx",
            currency="usd",
            provider="bridge",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountCreateResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_create(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_id` but received ''"):
            await async_client.fiat.accounts.with_raw_response.create(
                user_id="",
                account_owner_name="xxx",
                currency="usd",
                provider="bridge",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get(self, async_client: AsyncPrivyAPI) -> None:
        account = await async_client.fiat.accounts.get(
            user_id="user_id",
            provider="bridge",
        )
        assert_matches_type(AccountGetResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.fiat.accounts.with_raw_response.get(
            user_id="user_id",
            provider="bridge",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountGetResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.fiat.accounts.with_streaming_response.get(
            user_id="user_id",
            provider="bridge",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountGetResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_id` but received ''"):
            await async_client.fiat.accounts.with_raw_response.get(
                user_id="",
                provider="bridge",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Wallet", "AdditionalSigner"]


class AdditionalSigner(BaseModel):
    signer_id: str


class Wallet(BaseModel):
    id: str
    """Unique ID of the wallet.

    This will be the primary identifier when using the wallet in the future.
    """

    additional_signers: List[AdditionalSigner]
    """Additional signers for the wallet."""

    address: str
    """Address of the wallet."""

    chain_type: Literal["solana", "ethereum", "cosmos", "stellar", "sui"]
    """Chain type of the wallet. 'Ethereum' supports any EVM-compatible network."""

    created_at: float
    """Unix timestamp of when the wallet was created in milliseconds."""

    owner_id: str
    """The key quorum ID of the owner of the wallet."""

    policy_ids: List[str]
    """List of policy IDs for policies that are enforced on the wallet."""

    public_key: Optional[str] = None
    """
    The compressed, raw public key for the wallet along the chain cryptographic
    curve.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from privy import PrivyAPI, AsyncPrivyAPI
from tests.utils import assert_matches_type
from privy.types.fiat import OnrampCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOnramp:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: PrivyAPI) -> None:
        onramp = client.fiat.onramp.create(
            user_id="user_id",
            amount="x",
            destination={
                "chain": "ethereum",
                "currency": "usdc",
                "to_address": "to_address",
            },
            provider="bridge",
            source={
                "currency": "usd",
                "payment_rail": "sepa",
            },
        )
        assert_matches_type(OnrampCreateResponse, onramp, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: PrivyAPI) -> None:
        response = client.fiat.onramp.with_raw_response.create(
            user_id="user_id",
            amount="x",
            destination={
                "chain": "ethereum",
                "currency": "usdc",
                "to_address": "to_address",
            },
            provider="bridge",
            source={
                "currency": "usd",
                "payment_rail": "sepa",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        onramp = response.parse()
        assert_matches_type(OnrampCreateResponse, onramp, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: PrivyAPI) -> None:
        with client.fiat.onramp.with_streaming_response.create(
            user_id="user_id",
            amount="x",
            destination={
                "chain": "ethereum",
                "currency": "usdc",
                "to_address": "to_address",
            },
            provider="bridge",
            source={
                "currency": "usd",
                "payment_rail": "sepa",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            onramp = response.parse()
            assert_matches_type(OnrampCreateResponse, onramp, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_create(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_id` but received ''"):
            client.fiat.onramp.with_raw_response.create(
                user_id="",
                amount="x",
                destination={
                    "chain": "ethereum",
                    "currency": "usdc",
                    "to_address": "to_address",
                },
                provider="bridge",
                source={
                    "currency": "usd",
                    "payment_rail": "sepa",
                },
            )


class TestAsyncOnramp:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncPrivyAPI) -> None:
        onramp = await async_client.fiat.onramp.create(
            user_id="user_id",
            amount="x",
            destination={
                "chain": "ethereum",
                "currency": "usdc",
                "to_address": "to_address",
            },
            provider="bridge",
            source={
                "currency": "usd",
                "payment_rail": "sepa",
            },
        )
        assert_matches_type(OnrampCreateResponse, onramp, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.fiat.onramp.with_raw_response.create(
            user_id="user_id",
            amount="x",
            destination={
                "chain": "ethereum",
                "currency": "usdc",
                "to_address": "to_address",
            },
            provider="bridge",
            source={
                "currency": "usd",
                "payment_rail": "sepa",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        onramp = await response.parse()
        assert_matches_type(OnrampCreateResponse, onramp, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.fiat.onramp.with_streaming_response.create(
            user_id="user_id",
            amount="x",
            destination={
                "chain": "ethereum",
                "currency": "usdc",
                "to_address": "to_address",
            },
            provider="bridge",
            source={
                "currency": "usd",
                "payment_rail": "sepa",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            onramp = await response.parse()
            assert_matches_type(OnrampCreateResponse, onramp, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_create(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_id` but received ''"):
            await async_client.fiat.onramp.with_raw_response.create(
                user_id="",
                amount="x",
                destination={
                    "chain": "ethereum",
                    "currency": "usdc",
                    "to_address": "to_address",
                },
                provider="bridge",
                source={
                    "currency": "usd",
                    "payment_rail": "sepa",
                },
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["WalletCreateParams", "AdditionalSigner", "Owner"]


class WalletCreateParams(TypedDict, total=False):
    chain_type: Required[Literal["solana", "ethereum", "cosmos", "stellar", "sui"]]
    """Chain type of the wallet. "ethereum" supports any EVM-compatible network."""

    additional_signers: Iterable[AdditionalSigner]
    """Additional signers for the wallet."""

    owner: Optional[Owner]
    """The P-256 public key of the owner of the wallet.

    If you provide this, do not specify an owner_id as it will be generated
    automatically.
    """

    owner_id: Optional[str]
    """The key quorum ID to set as the owner of the wallet.

    If you provide this, do not specify an owner.
    """

    policy_ids: List[str]
    """List of policy IDs for policies that should be enforced on the wallet.

    Currently, only one policy is supported per wallet.
    """

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


class AdditionalSigner(TypedDict, total=False):
    override_policy_ids: Required[List[str]]

    signer_id: Required[str]


class Owner(TypedDict, total=False):
    public_key: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["WalletListParams"]


class WalletListParams(TypedDict, total=False):
    chain_type: Literal["solana", "ethereum", "cosmos", "stellar", "sui"]
    """Chain type of the wallet. 'Ethereum' supports any EVM-compatible network."""

    cursor: str

    limit: Optional[float]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["KeyQuorumUpdateParams"]


class KeyQuorumUpdateParams(TypedDict, total=False):
    public_keys: Required[List[str]]

    authorization_threshold: float

    display_name: str

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["AccountCreateResponse"]


class AccountCreateResponse(BaseModel):
    id: str

    account_type: str

    currency: str

    bank_name: Optional[str] = None

    last_4: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .user import User as User
from .policy import Policy as Policy
from .wallet import Wallet as Wallet
from .key_quorum import KeyQuorum as KeyQuorum
from .user_list_params import UserListParams as UserListParams
from .wallet_rpc_params import WalletRpcParams as WalletRpcParams
from .user_create_params import UserCreateParams as UserCreateParams
from .wallet_list_params import WalletListParams as WalletListParams
from .wallet_rpc_response import WalletRpcResponse as WalletRpcResponse
from .policy_create_params import PolicyCreateParams as PolicyCreateParams
from .policy_update_params import PolicyUpdateParams as PolicyUpdateParams
from .user_delete_response import UserDeleteResponse as UserDeleteResponse
from .wallet_create_params import WalletCreateParams as WalletCreateParams
from .wallet_update_params import WalletUpdateParams as WalletUpdateParams
from .fiat_configure_params import FiatConfigureParams as FiatConfigureParams
from .fiat_configure_response import FiatConfigureResponse as FiatConfigureResponse
from .key_quorum_create_params import KeyQuorumCreateParams as KeyQuorumCreateParams
from .key_quorum_update_params import KeyQuorumUpdateParams as KeyQuorumUpdateParams
from .transaction_get_response import TransactionGetResponse as TransactionGetResponse
from .wallet_authenticate_with_jwt_params import WalletAuthenticateWithJwtParams as WalletAuthenticateWithJwtParams
from .user_create_custom_metadata_response import UserCreateCustomMetadataResponse as UserCreateCustomMetadataResponse
from .wallet_authenticate_with_jwt_response import (
    WalletAuthenticateWithJwtResponse as WalletAuthenticateWithJwtResponse,
)
from .wallet_create_wallets_with_recovery_params import (
    WalletCreateWalletsWithRecoveryParams as WalletCreateWalletsWithRecoveryParams,
)
from .wallet_create_wallets_with_recovery_response import (
    WalletCreateWalletsWithRecoveryResponse as WalletCreateWalletsWithRecoveryResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["OfframpCreateResponse", "DepositInstructions"]


class DepositInstructions(BaseModel):
    amount: str

    chain: Literal["ethereum", "base", "arbitrum", "polygon", "optimism"]

    currency: Literal["usdc"]

    from_address: str

    to_address: str


class OfframpCreateResponse(BaseModel):
    id: str

    deposit_instructions: DepositInstructions

    status: Literal[
        "awaiting_funds",
        "in_review",
        "funds_received",
        "payment_submitted",
        "payment_processed",
        "canceled",
        "error",
        "undeliverable",
        "returned",
        "refunded",
    ]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.fiat import account_get_params, account_create_params
from ..._base_client import make_request_options
from ...types.fiat.account_get_response import AccountGetResponse
from ...types.fiat.account_create_response import AccountCreateResponse

__all__ = ["AccountsResource", "AsyncAccountsResource"]


class AccountsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return AccountsResourceWithStreamingResponse(self)

    def create(
        self,
        user_id: str,
        *,
        account_owner_name: str,
        currency: Literal["usd", "eur"],
        provider: Literal["bridge", "bridge-sandbox"],
        account: account_create_params.Account | NotGiven = NOT_GIVEN,
        address: account_create_params.Address | NotGiven = NOT_GIVEN,
        bank_name: str | NotGiven = NOT_GIVEN,
        first_name: str | NotGiven = NOT_GIVEN,
        iban: account_create_params.Iban | NotGiven = NOT_GIVEN,
        last_name: str | NotGiven = NOT_GIVEN,
        swift: account_create_params.Swift | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountCreateResponse:
        """
        Sets up external bank account object for the user through the configured default
        provider. Requires the user to already be KYC'ed.

        Args:
          user_id: The ID of the user to create the fiat account for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._post(
            f"/v1/users/{user_id}/fiat/accounts",
            body=maybe_transform(
                {
                    "account_owner_name": account_owner_name,
                    "currency": currency,
                    "provider": provider,
                    "account": account,
                    "address": address,
                    "bank_name": bank_name,
                    "first_name": first_name,
                    "iban": iban,
                    "last_name": last_name,
                    "swift": swift,
                },
                account_create_params.AccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountCreateResponse,
        )

    def get(
        self,
        user_id: str,
        *,
        provider: Literal["bridge", "bridge-sandbox"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountGetResponse:
        """
        Returns the IDs of all external fiat accounts (used for offramping) for the user

        Args:
          user_id: The ID of the user to get fiat accounts for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            f"/v1/users/{user_id}/fiat/accounts",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"provider": provider}, account_get_params.AccountGetParams),
            ),
            cast_to=AccountGetResponse,
        )


class AsyncAccountsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return AsyncAccountsResourceWithStreamingResponse(self)

    async def create(
        self,
        user_id: str,
        *,
        account_owner_name: str,
        currency: Literal["usd", "eur"],
        provider: Literal["bridge", "bridge-sandbox"],
        account: account_create_params.Account | NotGiven = NOT_GIVEN,
        address: account_create_params.Address | NotGiven = NOT_GIVEN,
        bank_name: str | NotGiven = NOT_GIVEN,
        first_name: str | NotGiven = NOT_GIVEN,
        iban: account_create_params.Iban | NotGiven = NOT_GIVEN,
        last_name: str | NotGiven = NOT_GIVEN,
        swift: account_create_params.Swift | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountCreateResponse:
        """
        Sets up external bank account object for the user through the configured default
        provider. Requires the user to already be KYC'ed.

        Args:
          user_id: The ID of the user to create the fiat account for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._post(
            f"/v1/users/{user_id}/fiat/accounts",
            body=await async_maybe_transform(
                {
                    "account_owner_name": account_owner_name,
                    "currency": currency,
                    "provider": provider,
                    "account": account,
                    "address": address,
                    "bank_name": bank_name,
                    "first_name": first_name,
                    "iban": iban,
                    "last_name": last_name,
                    "swift": swift,
                },
                account_create_params.AccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountCreateResponse,
        )

    async def get(
        self,
        user_id: str,
        *,
        provider: Literal["bridge", "bridge-sandbox"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountGetResponse:
        """
        Returns the IDs of all external fiat accounts (used for offramping) for the user

        Args:
          user_id: The ID of the user to get fiat accounts for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            f"/v1/users/{user_id}/fiat/accounts",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"provider": provider}, account_get_params.AccountGetParams),
            ),
            cast_to=AccountGetResponse,
        )


class AccountsResourceWithRawResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.create = to_raw_response_wrapper(
            accounts.create,
        )
        self.get = to_raw_response_wrapper(
            accounts.get,
        )


class AsyncAccountsResourceWithRawResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.create = async_to_raw_response_wrapper(
            accounts.create,
        )
        self.get = async_to_raw_response_wrapper(
            accounts.get,
        )


class AccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.create = to_streamed_response_wrapper(
            accounts.create,
        )
        self.get = to_streamed_response_wrapper(
            accounts.get,
        )


class AsyncAccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.create = async_to_streamed_response_wrapper(
            accounts.create,
        )
        self.get = async_to_streamed_response_wrapper(
            accounts.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["WalletAuthenticateWithJwtParams"]


class WalletAuthenticateWithJwtParams(TypedDict, total=False):
    encryption_type: Required[Literal["HPKE"]]
    """The encryption type for the authentication response.

    Currently only supports HPKE.
    """

    recipient_public_key: Required[str]
    """Base64-encoded public key of the recipient who will decrypt the session key.

    This key must be generated securely and kept confidential.
    """

    user_jwt: Required[str]
    """The user's JWT, to be used to authenticate the user."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BalanceGetParams"]


class BalanceGetParams(TypedDict, total=False):
    asset: Required[Union[Literal["usdc", "eth"], List[Literal["usdc", "eth"]]]]

    chain: Required[
        Union[
            Literal["ethereum", "arbitrum", "base", "linea", "optimism", "zksync_era"],
            List[Literal["ethereum", "arbitrum", "base", "linea", "optimism", "zksync_era"]],
        ]
    ]

    include_currency: Literal["usd"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from privy import PrivyAPI, AsyncPrivyAPI
from privy.types import (
    Wallet,
    WalletRpcResponse,
    WalletAuthenticateWithJwtResponse,
    WalletCreateWalletsWithRecoveryResponse,
)
from tests.utils import assert_matches_type
from privy.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWallets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: PrivyAPI) -> None:
        wallet = client.wallets.create(
            chain_type="ethereum",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: PrivyAPI) -> None:
        wallet = client.wallets.create(
            chain_type="ethereum",
            additional_signers=[
                {
                    "override_policy_ids": ["string"],
                    "signer_id": "signer_id",
                }
            ],
            owner={"public_key": "public_key"},
            owner_id="owner_id",
            policy_ids=["xxxxxxxxxxxxxxxxxxxxxxxx"],
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.create(
            chain_type="ethereum",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.create(
            chain_type="ethereum",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(Wallet, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: PrivyAPI) -> None:
        wallet = client.wallets.update(
            wallet_id="wallet_id",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: PrivyAPI) -> None:
        wallet = client.wallets.update(
            wallet_id="wallet_id",
            additional_signers=[
                {
                    "override_policy_ids": ["string"],
                    "signer_id": "signer_id",
                }
            ],
            owner={"public_key": "public_key"},
            owner_id="owner_id",
            policy_ids=["tb54eps4z44ed0jepousxi4n"],
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.update(
            wallet_id="wallet_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.update(
            wallet_id="wallet_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(Wallet, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.update(
                wallet_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: PrivyAPI) -> None:
        wallet = client.wallets.list()
        assert_matches_type(SyncCursor[Wallet], wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: PrivyAPI) -> None:
        wallet = client.wallets.list(
            chain_type="solana",
            cursor="x",
            limit=100,
        )
        assert_matches_type(SyncCursor[Wallet], wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(SyncCursor[Wallet], wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(SyncCursor[Wallet], wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_authenticate_with_jwt(self, client: PrivyAPI) -> None:
        wallet = client.wallets.authenticate_with_jwt(
            encryption_type="HPKE",
            recipient_public_key="DAQcDQgAEx4aoeD72yykviK+fckqE2CItVIGn1rCnvCXZ1HgpOcMEMialRmTrqIK4oZlYd1",
            user_jwt="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWUsImlhdCI6MTUxNjIzOTAyMn0.KMUFsIDTnFmyG3nMiGM6H9FNFUROf3wh7SmqJp-QV30",
        )
        assert_matches_type(WalletAuthenticateWithJwtResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_authenticate_with_jwt(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.authenticate_with_jwt(
            encryption_type="HPKE",
            recipient_public_key="DAQcDQgAEx4aoeD72yykviK+fckqE2CItVIGn1rCnvCXZ1HgpOcMEMialRmTrqIK4oZlYd1",
            user_jwt="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWUsImlhdCI6MTUxNjIzOTAyMn0.KMUFsIDTnFmyG3nMiGM6H9FNFUROf3wh7SmqJp-QV30",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletAuthenticateWithJwtResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_authenticate_with_jwt(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.authenticate_with_jwt(
            encryption_type="HPKE",
            recipient_public_key="DAQcDQgAEx4aoeD72yykviK+fckqE2CItVIGn1rCnvCXZ1HgpOcMEMialRmTrqIK4oZlYd1",
            user_jwt="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWUsImlhdCI6MTUxNjIzOTAyMn0.KMUFsIDTnFmyG3nMiGM6H9FNFUROf3wh7SmqJp-QV30",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletAuthenticateWithJwtResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_create_wallets_with_recovery(self, client: PrivyAPI) -> None:
        wallet = client.wallets.create_wallets_with_recovery(
            primary_signer={"subject_id": "cm7oxq1el000e11o8iwp7d0d0"},
            recovery_user={
                "linked_accounts": [
                    {
                        "address": "john@doe.com",
                        "type": "email",
                    }
                ]
            },
            wallets=[{"chain_type": "ethereum"}, {"chain_type": "solana"}],
        )
        assert_matches_type(WalletCreateWalletsWithRecoveryResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_wallets_with_recovery(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.create_wallets_with_recovery(
            primary_signer={"subject_id": "cm7oxq1el000e11o8iwp7d0d0"},
            recovery_user={
                "linked_accounts": [
                    {
                        "address": "john@doe.com",
                        "type": "email",
                    }
                ]
            },
            wallets=[{"chain_type": "ethereum"}, {"chain_type": "solana"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletCreateWalletsWithRecoveryResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_wallets_with_recovery(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.create_wallets_with_recovery(
            primary_signer={"subject_id": "cm7oxq1el000e11o8iwp7d0d0"},
            recovery_user={
                "linked_accounts": [
                    {
                        "address": "john@doe.com",
                        "type": "email",
                    }
                ]
            },
            wallets=[{"chain_type": "ethereum"}, {"chain_type": "solana"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletCreateWalletsWithRecoveryResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get(self, client: PrivyAPI) -> None:
        wallet = client.wallets.get(
            "wallet_id",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.get(
            "wallet_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.get(
            "wallet_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(Wallet, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.get(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_overload_1(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="eth_signTransaction",
            params={"transaction": {}},
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_with_all_params_overload_1(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="eth_signTransaction",
            params={
                "transaction": {
                    "chain_id": "string",
                    "data": "data",
                    "from": "from",
                    "gas_limit": "string",
                    "gas_price": "string",
                    "max_fee_per_gas": "string",
                    "max_priority_fee_per_gas": "string",
                    "nonce": "string",
                    "to": "to",
                    "type": 0,
                    "value": "string",
                }
            },
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_rpc_overload_1(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="eth_signTransaction",
            params={"transaction": {}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_rpc_overload_1(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="eth_signTransaction",
            params={"transaction": {}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_rpc_overload_1(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="eth_signTransaction",
                params={"transaction": {}},
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_overload_2(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="eth_sendTransaction",
            params={"transaction": {}},
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_with_all_params_overload_2(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="eth_sendTransaction",
            params={
                "transaction": {
                    "chain_id": "string",
                    "data": "data",
                    "from": "from",
                    "gas_limit": "string",
                    "gas_price": "string",
                    "max_fee_per_gas": "string",
                    "max_priority_fee_per_gas": "string",
                    "nonce": "string",
                    "to": "to",
                    "type": 0,
                    "value": "string",
                }
            },
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_rpc_overload_2(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="eth_sendTransaction",
            params={"transaction": {}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_rpc_overload_2(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="eth_sendTransaction",
            params={"transaction": {}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_rpc_overload_2(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.rpc(
                wallet_id="",
                caip2="-l-f12-k:_--l__36_",
                method="eth_sendTransaction",
                params={"transaction": {}},
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_overload_3(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="personal_sign",
            params={
                "encoding": "utf-8",
                "message": "message",
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_with_all_params_overload_3(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="personal_sign",
            params={
                "encoding": "utf-8",
                "message": "message",
            },
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_rpc_overload_3(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="personal_sign",
            params={
                "encoding": "utf-8",
                "message": "message",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_rpc_overload_3(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="personal_sign",
            params={
                "encoding": "utf-8",
                "message": "message",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_rpc_overload_3(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="personal_sign",
                params={
                    "encoding": "utf-8",
                    "message": "message",
                },
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_overload_4(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="eth_signTypedData_v4",
            params={
                "typed_data": {
                    "domain": {"foo": "bar"},
                    "message": {"foo": "bar"},
                    "primary_type": "primary_type",
                    "types": {
                        "foo": [
                            {
                                "name": "name",
                                "type": "type",
                            }
                        ]
                    },
                }
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_with_all_params_overload_4(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="eth_signTypedData_v4",
            params={
                "typed_data": {
                    "domain": {"foo": "bar"},
                    "message": {"foo": "bar"},
                    "primary_type": "primary_type",
                    "types": {
                        "foo": [
                            {
                                "name": "name",
                                "type": "type",
                            }
                        ]
                    },
                }
            },
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_rpc_overload_4(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="eth_signTypedData_v4",
            params={
                "typed_data": {
                    "domain": {"foo": "bar"},
                    "message": {"foo": "bar"},
                    "primary_type": "primary_type",
                    "types": {
                        "foo": [
                            {
                                "name": "name",
                                "type": "type",
                            }
                        ]
                    },
                }
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_rpc_overload_4(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="eth_signTypedData_v4",
            params={
                "typed_data": {
                    "domain": {"foo": "bar"},
                    "message": {"foo": "bar"},
                    "primary_type": "primary_type",
                    "types": {
                        "foo": [
                            {
                                "name": "name",
                                "type": "type",
                            }
                        ]
                    },
                }
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_rpc_overload_4(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="eth_signTypedData_v4",
                params={
                    "typed_data": {
                        "domain": {"foo": "bar"},
                        "message": {"foo": "bar"},
                        "primary_type": "primary_type",
                        "types": {
                            "foo": [
                                {
                                    "name": "name",
                                    "type": "type",
                                }
                            ]
                        },
                    }
                },
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_overload_5(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="secp256k1_sign",
            params={},
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_with_all_params_overload_5(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="secp256k1_sign",
            params={"hash": {}},
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_rpc_overload_5(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="secp256k1_sign",
            params={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_rpc_overload_5(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="secp256k1_sign",
            params={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_rpc_overload_5(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="secp256k1_sign",
                params={},
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_overload_6(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="signTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_with_all_params_overload_6(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="signTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
            address="address",
            chain_type="solana",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_rpc_overload_6(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="signTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_rpc_overload_6(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="signTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_rpc_overload_6(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="signTransaction",
                params={
                    "encoding": "base64",
                    "transaction": "transaction",
                },
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_overload_7(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="signAndSendTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_with_all_params_overload_7(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="signAndSendTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
            address="address",
            chain_type="solana",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_rpc_overload_7(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="signAndSendTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_rpc_overload_7(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="signAndSendTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_rpc_overload_7(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.rpc(
                wallet_id="",
                caip2="-l-f12-k:_--l__36_",
                method="signAndSendTransaction",
                params={
                    "encoding": "base64",
                    "transaction": "transaction",
                },
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_overload_8(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="signMessage",
            params={
                "encoding": "base64",
                "message": "message",
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_rpc_with_all_params_overload_8(self, client: PrivyAPI) -> None:
        wallet = client.wallets.rpc(
            wallet_id="wallet_id",
            method="signMessage",
            params={
                "encoding": "base64",
                "message": "message",
            },
            address="address",
            chain_type="solana",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_rpc_overload_8(self, client: PrivyAPI) -> None:
        response = client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="signMessage",
            params={
                "encoding": "base64",
                "message": "message",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_rpc_overload_8(self, client: PrivyAPI) -> None:
        with client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="signMessage",
            params={
                "encoding": "base64",
                "message": "message",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_rpc_overload_8(self, client: PrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="signMessage",
                params={
                    "encoding": "base64",
                    "message": "message",
                },
            )


class TestAsyncWallets:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.create(
            chain_type="ethereum",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.create(
            chain_type="ethereum",
            additional_signers=[
                {
                    "override_policy_ids": ["string"],
                    "signer_id": "signer_id",
                }
            ],
            owner={"public_key": "public_key"},
            owner_id="owner_id",
            policy_ids=["xxxxxxxxxxxxxxxxxxxxxxxx"],
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.create(
            chain_type="ethereum",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.create(
            chain_type="ethereum",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(Wallet, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.update(
            wallet_id="wallet_id",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.update(
            wallet_id="wallet_id",
            additional_signers=[
                {
                    "override_policy_ids": ["string"],
                    "signer_id": "signer_id",
                }
            ],
            owner={"public_key": "public_key"},
            owner_id="owner_id",
            policy_ids=["tb54eps4z44ed0jepousxi4n"],
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.update(
            wallet_id="wallet_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.update(
            wallet_id="wallet_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(Wallet, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.update(
                wallet_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.list()
        assert_matches_type(AsyncCursor[Wallet], wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.list(
            chain_type="solana",
            cursor="x",
            limit=100,
        )
        assert_matches_type(AsyncCursor[Wallet], wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(AsyncCursor[Wallet], wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(AsyncCursor[Wallet], wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_authenticate_with_jwt(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.authenticate_with_jwt(
            encryption_type="HPKE",
            recipient_public_key="DAQcDQgAEx4aoeD72yykviK+fckqE2CItVIGn1rCnvCXZ1HgpOcMEMialRmTrqIK4oZlYd1",
            user_jwt="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWUsImlhdCI6MTUxNjIzOTAyMn0.KMUFsIDTnFmyG3nMiGM6H9FNFUROf3wh7SmqJp-QV30",
        )
        assert_matches_type(WalletAuthenticateWithJwtResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_authenticate_with_jwt(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.authenticate_with_jwt(
            encryption_type="HPKE",
            recipient_public_key="DAQcDQgAEx4aoeD72yykviK+fckqE2CItVIGn1rCnvCXZ1HgpOcMEMialRmTrqIK4oZlYd1",
            user_jwt="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWUsImlhdCI6MTUxNjIzOTAyMn0.KMUFsIDTnFmyG3nMiGM6H9FNFUROf3wh7SmqJp-QV30",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletAuthenticateWithJwtResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_authenticate_with_jwt(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.authenticate_with_jwt(
            encryption_type="HPKE",
            recipient_public_key="DAQcDQgAEx4aoeD72yykviK+fckqE2CItVIGn1rCnvCXZ1HgpOcMEMialRmTrqIK4oZlYd1",
            user_jwt="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWUsImlhdCI6MTUxNjIzOTAyMn0.KMUFsIDTnFmyG3nMiGM6H9FNFUROf3wh7SmqJp-QV30",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletAuthenticateWithJwtResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_wallets_with_recovery(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.create_wallets_with_recovery(
            primary_signer={"subject_id": "cm7oxq1el000e11o8iwp7d0d0"},
            recovery_user={
                "linked_accounts": [
                    {
                        "address": "john@doe.com",
                        "type": "email",
                    }
                ]
            },
            wallets=[{"chain_type": "ethereum"}, {"chain_type": "solana"}],
        )
        assert_matches_type(WalletCreateWalletsWithRecoveryResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_wallets_with_recovery(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.create_wallets_with_recovery(
            primary_signer={"subject_id": "cm7oxq1el000e11o8iwp7d0d0"},
            recovery_user={
                "linked_accounts": [
                    {
                        "address": "john@doe.com",
                        "type": "email",
                    }
                ]
            },
            wallets=[{"chain_type": "ethereum"}, {"chain_type": "solana"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletCreateWalletsWithRecoveryResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_wallets_with_recovery(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.create_wallets_with_recovery(
            primary_signer={"subject_id": "cm7oxq1el000e11o8iwp7d0d0"},
            recovery_user={
                "linked_accounts": [
                    {
                        "address": "john@doe.com",
                        "type": "email",
                    }
                ]
            },
            wallets=[{"chain_type": "ethereum"}, {"chain_type": "solana"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletCreateWalletsWithRecoveryResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.get(
            "wallet_id",
        )
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.get(
            "wallet_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(Wallet, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.get(
            "wallet_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(Wallet, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.get(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_overload_1(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="eth_signTransaction",
            params={"transaction": {}},
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_with_all_params_overload_1(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="eth_signTransaction",
            params={
                "transaction": {
                    "chain_id": "string",
                    "data": "data",
                    "from": "from",
                    "gas_limit": "string",
                    "gas_price": "string",
                    "max_fee_per_gas": "string",
                    "max_priority_fee_per_gas": "string",
                    "nonce": "string",
                    "to": "to",
                    "type": 0,
                    "value": "string",
                }
            },
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_rpc_overload_1(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="eth_signTransaction",
            params={"transaction": {}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_rpc_overload_1(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="eth_signTransaction",
            params={"transaction": {}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_rpc_overload_1(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="eth_signTransaction",
                params={"transaction": {}},
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_overload_2(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="eth_sendTransaction",
            params={"transaction": {}},
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_with_all_params_overload_2(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="eth_sendTransaction",
            params={
                "transaction": {
                    "chain_id": "string",
                    "data": "data",
                    "from": "from",
                    "gas_limit": "string",
                    "gas_price": "string",
                    "max_fee_per_gas": "string",
                    "max_priority_fee_per_gas": "string",
                    "nonce": "string",
                    "to": "to",
                    "type": 0,
                    "value": "string",
                }
            },
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_rpc_overload_2(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="eth_sendTransaction",
            params={"transaction": {}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_rpc_overload_2(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="eth_sendTransaction",
            params={"transaction": {}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_rpc_overload_2(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.rpc(
                wallet_id="",
                caip2="-l-f12-k:_--l__36_",
                method="eth_sendTransaction",
                params={"transaction": {}},
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_overload_3(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="personal_sign",
            params={
                "encoding": "utf-8",
                "message": "message",
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_with_all_params_overload_3(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="personal_sign",
            params={
                "encoding": "utf-8",
                "message": "message",
            },
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_rpc_overload_3(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="personal_sign",
            params={
                "encoding": "utf-8",
                "message": "message",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_rpc_overload_3(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="personal_sign",
            params={
                "encoding": "utf-8",
                "message": "message",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_rpc_overload_3(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="personal_sign",
                params={
                    "encoding": "utf-8",
                    "message": "message",
                },
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_overload_4(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="eth_signTypedData_v4",
            params={
                "typed_data": {
                    "domain": {"foo": "bar"},
                    "message": {"foo": "bar"},
                    "primary_type": "primary_type",
                    "types": {
                        "foo": [
                            {
                                "name": "name",
                                "type": "type",
                            }
                        ]
                    },
                }
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_with_all_params_overload_4(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="eth_signTypedData_v4",
            params={
                "typed_data": {
                    "domain": {"foo": "bar"},
                    "message": {"foo": "bar"},
                    "primary_type": "primary_type",
                    "types": {
                        "foo": [
                            {
                                "name": "name",
                                "type": "type",
                            }
                        ]
                    },
                }
            },
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_rpc_overload_4(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="eth_signTypedData_v4",
            params={
                "typed_data": {
                    "domain": {"foo": "bar"},
                    "message": {"foo": "bar"},
                    "primary_type": "primary_type",
                    "types": {
                        "foo": [
                            {
                                "name": "name",
                                "type": "type",
                            }
                        ]
                    },
                }
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_rpc_overload_4(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="eth_signTypedData_v4",
            params={
                "typed_data": {
                    "domain": {"foo": "bar"},
                    "message": {"foo": "bar"},
                    "primary_type": "primary_type",
                    "types": {
                        "foo": [
                            {
                                "name": "name",
                                "type": "type",
                            }
                        ]
                    },
                }
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_rpc_overload_4(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="eth_signTypedData_v4",
                params={
                    "typed_data": {
                        "domain": {"foo": "bar"},
                        "message": {"foo": "bar"},
                        "primary_type": "primary_type",
                        "types": {
                            "foo": [
                                {
                                    "name": "name",
                                    "type": "type",
                                }
                            ]
                        },
                    }
                },
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_overload_5(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="secp256k1_sign",
            params={},
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_with_all_params_overload_5(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="secp256k1_sign",
            params={"hash": {}},
            address="address",
            chain_type="ethereum",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_rpc_overload_5(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="secp256k1_sign",
            params={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_rpc_overload_5(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="secp256k1_sign",
            params={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_rpc_overload_5(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="secp256k1_sign",
                params={},
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_overload_6(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="signTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_with_all_params_overload_6(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="signTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
            address="address",
            chain_type="solana",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_rpc_overload_6(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="signTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_rpc_overload_6(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="signTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_rpc_overload_6(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="signTransaction",
                params={
                    "encoding": "base64",
                    "transaction": "transaction",
                },
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_overload_7(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="signAndSendTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_with_all_params_overload_7(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="signAndSendTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
            address="address",
            chain_type="solana",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_rpc_overload_7(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="signAndSendTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_rpc_overload_7(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            caip2="-l-f12-k:_--l__36_",
            method="signAndSendTransaction",
            params={
                "encoding": "base64",
                "transaction": "transaction",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_rpc_overload_7(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.rpc(
                wallet_id="",
                caip2="-l-f12-k:_--l__36_",
                method="signAndSendTransaction",
                params={
                    "encoding": "base64",
                    "transaction": "transaction",
                },
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_overload_8(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="signMessage",
            params={
                "encoding": "base64",
                "message": "message",
            },
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_rpc_with_all_params_overload_8(self, async_client: AsyncPrivyAPI) -> None:
        wallet = await async_client.wallets.rpc(
            wallet_id="wallet_id",
            method="signMessage",
            params={
                "encoding": "base64",
                "message": "message",
            },
            address="address",
            chain_type="solana",
            privy_authorization_signature="privy-authorization-signature",
        )
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_rpc_overload_8(self, async_client: AsyncPrivyAPI) -> None:
        response = await async_client.wallets.with_raw_response.rpc(
            wallet_id="wallet_id",
            method="signMessage",
            params={
                "encoding": "base64",
                "message": "message",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        wallet = await response.parse()
        assert_matches_type(WalletRpcResponse, wallet, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_rpc_overload_8(self, async_client: AsyncPrivyAPI) -> None:
        async with async_client.wallets.with_streaming_response.rpc(
            wallet_id="wallet_id",
            method="signMessage",
            params={
                "encoding": "base64",
                "message": "message",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            wallet = await response.parse()
            assert_matches_type(WalletRpcResponse, wallet, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_rpc_overload_8(self, async_client: AsyncPrivyAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `wallet_id` but received ''"):
            await async_client.wallets.with_raw_response.rpc(
                wallet_id="",
                method="signMessage",
                params={
                    "encoding": "base64",
                    "message": "message",
                },
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.wallets import balance_get_params
from ...types.wallets.balance_get_response import BalanceGetResponse

__all__ = ["BalanceResource", "AsyncBalanceResource"]


class BalanceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BalanceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return BalanceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BalanceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return BalanceResourceWithStreamingResponse(self)

    def get(
        self,
        wallet_id: str,
        *,
        asset: Union[Literal["usdc", "eth"], List[Literal["usdc", "eth"]]],
        chain: Union[
            Literal["ethereum", "arbitrum", "base", "linea", "optimism", "zksync_era"],
            List[Literal["ethereum", "arbitrum", "base", "linea", "optimism", "zksync_era"]],
        ],
        include_currency: Literal["usd"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BalanceGetResponse:
        """
        Get the balance of a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not wallet_id:
            raise ValueError(f"Expected a non-empty value for `wallet_id` but received {wallet_id!r}")
        return self._get(
            f"/v1/wallets/{wallet_id}/balance",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asset": asset,
                        "chain": chain,
                        "include_currency": include_currency,
                    },
                    balance_get_params.BalanceGetParams,
                ),
            ),
            cast_to=BalanceGetResponse,
        )


class AsyncBalanceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBalanceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/privy-io/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncBalanceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBalanceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/privy-io/python-sdk#with_streaming_response
        """
        return AsyncBalanceResourceWithStreamingResponse(self)

    async def get(
        self,
        wallet_id: str,
        *,
        asset: Union[Literal["usdc", "eth"], List[Literal["usdc", "eth"]]],
        chain: Union[
            Literal["ethereum", "arbitrum", "base", "linea", "optimism", "zksync_era"],
            List[Literal["ethereum", "arbitrum", "base", "linea", "optimism", "zksync_era"]],
        ],
        include_currency: Literal["usd"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BalanceGetResponse:
        """
        Get the balance of a wallet by wallet ID.

        Args:
          wallet_id: ID of the wallet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not wallet_id:
            raise ValueError(f"Expected a non-empty value for `wallet_id` but received {wallet_id!r}")
        return await self._get(
            f"/v1/wallets/{wallet_id}/balance",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asset": asset,
                        "chain": chain,
                        "include_currency": include_currency,
                    },
                    balance_get_params.BalanceGetParams,
                ),
            ),
            cast_to=BalanceGetResponse,
        )


class BalanceResourceWithRawResponse:
    def __init__(self, balance: BalanceResource) -> None:
        self._balance = balance

        self.get = to_raw_response_wrapper(
            balance.get,
        )


class AsyncBalanceResourceWithRawResponse:
    def __init__(self, balance: AsyncBalanceResource) -> None:
        self._balance = balance

        self.get = async_to_raw_response_wrapper(
            balance.get,
        )


class BalanceResourceWithStreamingResponse:
    def __init__(self, balance: BalanceResource) -> None:
        self._balance = balance

        self.get = to_streamed_response_wrapper(
            balance.get,
        )


class AsyncBalanceResourceWithStreamingResponse:
    def __init__(self, balance: AsyncBalanceResource) -> None:
        self._balance = balance

        self.get = async_to_streamed_response_wrapper(
            balance.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "PolicyUpdateParams",
    "Rule",
    "RuleCondition",
    "RuleConditionEthereumTransaction",
    "RuleConditionEthereumCalldata",
    "RuleConditionEthereumTypedDataDomain",
    "RuleConditionEthereumTypedDataMessage",
    "RuleConditionEthereumTypedDataMessageTypedData",
    "RuleConditionEthereumTypedDataMessageTypedDataType",
    "RuleConditionSolanaProgramInstruction",
    "RuleConditionSolanaSystemProgramInstruction",
    "RuleConditionSolanaTokenProgramInstruction",
]


class PolicyUpdateParams(TypedDict, total=False):
    name: str
    """Name to assign to policy."""

    rules: Iterable[Rule]
    """The rules that apply to each method the policy covers."""

    privy_authorization_signature: Annotated[str, PropertyInfo(alias="privy-authorization-signature")]
    """Request authorization signature.

    If multiple signatures are required, they should be comma separated.
    """


class RuleConditionEthereumTransaction(TypedDict, total=False):
    field: Required[Literal["to", "value"]]

    field_source: Required[Literal["ethereum_transaction"]]

    operator: Required[Literal["eq", "gt", "gte", "lt", "lte", "in"]]

    value: Required[Union[str, List[str]]]


class RuleConditionEthereumCalldata(TypedDict, total=False):
    abi: Required[object]

    field: Required[str]

    field_source: Required[Literal["ethereum_calldata"]]

    operator: Required[Literal["eq", "gt", "gte", "lt", "lte", "in"]]

    value: Required[Union[str, List[str]]]


class RuleConditionEthereumTypedDataDomain(TypedDict, total=False):
    field: Required[Literal["chainId", "verifyingContract"]]

    field_source: Required[Literal["ethereum_typed_data_domain"]]

    operator: Required[Literal["eq", "gt", "gte", "lt", "lte", "in"]]

    value: Required[Union[str, List[str]]]


class RuleConditionEthereumTypedDataMessageTypedDataType(TypedDict, total=False):
    name: Required[str]

    type: Required[str]


class RuleConditionEthereumTypedDataMessageTypedData(TypedDict, total=False):
    primary_type: Required[str]

    types: Required[Dict[str, Iterable[RuleConditionEthereumTypedDataMessageTypedDataType]]]


class RuleConditionEthereumTypedDataMessage(TypedDict, total=False):
    field: Required[str]

    field_source: Required[Literal["ethereum_typed_data_message"]]

    operator: Required[Literal["eq", "gt", "gte", "lt", "lte", "in"]]

    typed_data: Required[RuleConditionEthereumTypedDataMessageTypedData]

    value: Required[Union[str, List[str]]]


class RuleConditionSolanaProgramInstruction(TypedDict, total=False):
    field: Required[Literal["programId"]]

    field_source: Required[Literal["solana_program_instruction"]]

    operator: Required[Literal["eq", "gt", "gte", "lt", "lte", "in"]]

    value: Required[Union[str, List[str]]]


class RuleConditionSolanaSystemProgramInstruction(TypedDict, total=False):
    field: Required[Literal["instructionName", "Transfer.from", "Transfer.to", "Transfer.lamports"]]

    field_source: Required[Literal["solana_system_program_instruction"]]

    operator: Required[Literal["eq", "gt", "gte", "lt", "lte", "in"]]

    value: Required[Union[str, List[str]]]


class RuleConditionSolanaTokenProgramInstruction(TypedDict, total=False):
    field: Required[
        Literal[
            "instructionName",
            "TransferChecked.source",
            "TransferChecked.destination",
            "TransferChecked.authority",
            "TransferChecked.amount",
            "TransferChecked.mint",
        ]
    ]

    field_source: Required[Literal["solana_token_program_instruction"]]

    operator: Required[Literal["eq", "gt", "gte", "lt", "lte", "in"]]

    value: Required[Union[str, List[str]]]


RuleCondition: TypeAlias = Union[
    RuleConditionEthereumTransaction,
    RuleConditionEthereumCalldata,
    RuleConditionEthereumTypedDataDomain,
    RuleConditionEthereumTypedDataMessage,
    RuleConditionSolanaProgramInstruction,
    RuleConditionSolanaSystemProgramInstruction,
    RuleConditionSolanaTokenProgramInstruction,
]


class Rule(TypedDict, total=False):
    action: Required[Literal["ALLOW", "DENY"]]
    """Action to take if the conditions are true."""

    conditions: Required[Iterable[RuleCondition]]
    """
    An unordered set of boolean conditions that define the action the rule allows or
    denies.
    """

    method: Required[
        Literal[
            "eth_sendTransaction",
            "eth_signTransaction",
            "eth_signTypedData_v4",
            "signTransaction",
            "signAndSendTransaction",
            "exportPrivateKey",
            "*",
        ]
    ]
    """Method the rule applies to."""

    name: Required[str]
    """Name to assign to rule."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "WalletRpcResponse",
    "SignTransaction",
    "SignTransactionData",
    "SignAndSendTransaction",
    "SignAndSendTransactionData",
    "SignAndSendTransactionError",
    "SignMessage",
    "SignMessageData",
    "EthSignTransaction",
    "EthSignTransactionData",
    "EthSendTransaction",
    "EthSendTransactionData",
    "EthSendTransactionError",
    "PersonalSign",
    "PersonalSignData",
    "EthSignTypedDataV4",
    "EthSignTypedDataV4Data",
    "Secp256k1Sign",
    "Secp256k1SignData",
]


class SignTransactionData(BaseModel):
    encoding: Literal["base64"]

    signed_transaction: str


class SignTransaction(BaseModel):
    data: SignTransactionData

    method: Literal["signTransaction"]


class SignAndSendTransactionData(BaseModel):
    caip2: str

    hash: str

    transaction_id: Optional[str] = None


class SignAndSendTransactionError(BaseModel):
    code: str

    message: str


class SignAndSendTransaction(BaseModel):
    method: Literal["signAndSendTransaction"]

    data: Optional[SignAndSendTransactionData] = None

    error: Optional[SignAndSendTransactionError] = None


class SignMessageData(BaseModel):
    encoding: Literal["base64"]

    signature: str


class SignMessage(BaseModel):
    data: SignMessageData

    method: Literal["signMessage"]


class EthSignTransactionData(BaseModel):
    encoding: Literal["rlp"]

    signed_transaction: str


class EthSignTransaction(BaseModel):
    data: EthSignTransactionData

    method: Literal["eth_signTransaction"]


class EthSendTransactionData(BaseModel):
    caip2: str

    hash: str

    transaction_id: Optional[str] = None


class EthSendTransactionError(BaseModel):
    code: str

    message: str


class EthSendTransaction(BaseModel):
    method: Literal["eth_sendTransaction"]

    data: Optional[EthSendTransactionData] = None

    error: Optional[EthSendTransactionError] = None


class PersonalSignData(BaseModel):
    encoding: Literal["hex"]

    signature: str


class PersonalSign(BaseModel):
    data: PersonalSignData

    method: Literal["personal_sign"]


class EthSignTypedDataV4Data(BaseModel):
    encoding: Literal["hex"]

    signature: str


class EthSignTypedDataV4(BaseModel):
    data: EthSignTypedDataV4Data

    method: Literal["eth_signTypedData_v4"]


class Secp256k1SignData(BaseModel):
    encoding: Literal["hex"]

    signature: str


class Secp256k1Sign(BaseModel):
    data: Secp256k1SignData

    method: Literal["secp256k1_sign"]


WalletRpcResponse: TypeAlias = Union[
    SignTransaction,
    SignAndSendTransaction,
    SignMessage,
    EthSignTransaction,
    EthSendTransaction,
    PersonalSign,
    EthSignTypedDataV4,
    Secp256k1Sign,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TransactionGetResponse"]


class TransactionGetResponse(BaseModel):
    id: str

    caip2: str

    created_at: float

    status: Literal["broadcasted", "confirmed", "execution_reverted", "failed"]

    transaction_hash: Optional[str] = None

    wallet_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["OfframpCreateParams", "Destination", "Source"]


class OfframpCreateParams(TypedDict, total=False):
    amount: Required[str]

    destination: Required[Destination]

    provider: Required[Literal["bridge", "bridge-sandbox"]]

    source: Required[Source]


class Destination(TypedDict, total=False):
    currency: Required[Literal["usd", "eur"]]

    external_account_id: Required[str]

    payment_rail: Required[Literal["sepa", "ach_push", "wire"]]


class Source(TypedDict, total=False):
    chain: Required[Literal["ethereum", "base", "arbitrum", "polygon", "optimism"]]

    currency: Required[Literal["usdc"]]

    from_address: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["TransactionGetParams"]


class TransactionGetParams(TypedDict, total=False):
    asset: Required[Union[Literal["usdc", "eth"], List[Literal["usdc", "eth"]]]]

    chain: Required[Literal["base"]]

    cursor: str

    limit: Optional[float]
